import sys
import os
from pathlib import Path
from typing import Dict, Any
import requests

modules_path = '../../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import network_constants as nc
from security.token_manager import TokenManager

def add_by_layers():
    use_case = "smart-energy"

    layers = [
        {
            "use_case": use_case,
            "name": "Solar_Production_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Solar_Production_kWh"
            ],
            "cluster_properties": [
                "Solar_Production_kWh"
            ]
        },
        {
            "use_case": use_case,
            "name": "Energy_Consumption_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Energy_Consumption_kWh"
            ],
            "cluster_properties": [
                "Energy_Consumption_kWh"
            ]
        },{
            "use_case": use_case,
            "name": "Heating_Consumption_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Heating_Consumption_kWh"
            ],
            "cluster_properties": [
                "Heating_Consumption_kWh"
            ]
        },{
            "use_case": use_case,
            "name": "Price_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Price_AUD_MWh"
            ],
            "cluster_properties": [
                "Price_AUD_MWh"
            ]
        },{
            "use_case": use_case,
            "name": "Demand_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Total_Demand_MWh"
            ],
            "cluster_properties": [
                "Total_Demand_MWh"
            ]
        },{
            "use_case": use_case,
            "name": "Position_Layer",
            "properties": [
                "UniqueID",
                "Customer",
                "Postcode",
                "Timestamp",
                "Latitude", 
                "Longitude"
            ],
            "cluster_properties": [
                "Latitude",
                "Longitude",
            ]
        },
    ]

    jwt = TokenManager.getInstance().getToken()

    for layer in layers:
        url = f"https://articonf1.itec.aau.at:30420/api/layers"

        response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = layer
        )

        if response.status_code == 200:
            print(f"Response: {response.status_code}")
        else:
            print(f"Response: {response.status_code}: {response.text}")

def add_by_schema():
    '''
    take the columns and add the mappings at the server
    replace all "/"'s in the internal representation with a "_"
    '''
    jwt = TokenManager.getInstance().getToken()
    print(f"jwt token: {jwt}")

    use_case = "smart-energy"
    columns = ["Customer", "Postcode", "Timestamp", "Solar_Production_kWh", "Energy_Consumption_kWh", "Heating_Consumption_kWh", "Price_AUD/MWh", "Total_Demand_MWh", "Latitude", "Longitude"]
    columns = [(c.replace("/", "_"), c) for c in columns]
    columns.append(("UniqueID", "Customer+Postcode+Timestamp"))
    
    url = f"https://articonf1.itec.aau.at:30420/api/use-cases/{use_case}/schema"

    response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = columns
        )

    if response.status_code == 200:
        print(f"Response: {response.status_code}")
    else:
        print(f"Response: {response.status_code}: {response.text}")
    
if __name__ == "__main__":
    add_by_schema()
    add_by_layers()