#global imports
from db.entities.schema import Schema
from db.entities.layer_adapter import LayerAdapter
from db.schema_repository import SchemaRepository
from db.use_case_repository import UseCaseRepository
from services.layer_adapter_service import LayerAdapterService

import json
from flask import Response, request

use_case_repository = UseCaseRepository()
schema_repository = SchemaRepository()

def all():
    return [schema.to_serializable_dict() for schema in schema_repository.all()]

def add_complete(use_case: str):
    data = request.json

    if "mappings" not in data:
        return Response(status=400, response="Missing fields. Required fields: (mappings).")
    
    reference = schema_repository.get_for_use_case(use_case)
    if reference != None and len(reference.mappings) > 0:
        return Response(status=400, response="Schema already exists.")
    
    schema_repository.delete_for_use_case(use_case)
    data["use_case"] = use_case
    schema_repository.add(Schema.from_serializable_dict(data))

    return Response(status=200)

def get_for_use_case(use_case:str):
    use_case_repository.put(use_case)
    schema = schema_repository.get_for_use_case(use_case)

    if schema != None:
        return Response(status=200, response=json.dumps(schema.to_serializable_dict()))
    
    return Response(status=404, response=f"Schema {use_case} does not exist")

def delete_for_use_case(use_case:str):
    schema_repository.delete_for_use_case(use_case)

    return Response(status=200)

def add_mapping(use_case:str):
    use_case_repository.put(use_case)
    data = request.json

    if "internal" not in data or "external" not in data:
        return Response(status=400, response=f"Field missing! Fields required: (internal, external)")
    
    schema = schema_repository.get_for_use_case(use_case)

    if schema == None:
        print("schema not there, creating it...")
        schema = schema_repository.put(use_case)
    
    schema.add_mapping(data["internal"], data["external"])
    schema_repository.update(schema)

    return Response(status=200)